/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.web.gui.controladores.sendaAmigo;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Cliente;
import com.rjconsultores.ventaboletos.entidad.TipoIdentificacion;
import com.rjconsultores.ventaboletos.service.TipoIdentificacionService;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderCliente;
import com.trg.search.Filter;

@Controller("busquedaClienteController")
@Scope("prototype")
public class BusquedaClienteController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	@Autowired
	private transient PagedListWrapper<Cliente> plwCliente;
	
	@Autowired
	TipoIdentificacionService tipoIdentificacionService;
	
	private MyListbox clienteList;
	private Paging pagingCliente;
	private MyTextbox txtNombCliente;
	private MyTextbox txtApelidoPaterno;
	private MyTextbox txtApelidoMaterno;
	private MyTextbox txtNumRfc;
	private MyTextbox txtNumFidelidade;
	private MyTextbox txtDesccorreo;

	private List<TipoIdentificacion> lsTipoIdentificacion;
	private Combobox cmbTipoIdentificacion;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsTipoIdentificacion = tipoIdentificacionService.obtenerTodos();
		
		super.doAfterCompose(comp);

		clienteList.setItemRenderer(new RenderCliente());
		clienteList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				Cliente c = (Cliente) clienteList.getSelected();
				verCliente(c);
			}
		});

		refreshLista();
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verCliente(Cliente c) {
		if (c == null) {
			return;
		}

		Map args = new HashMap();
		args.put("cliente", c);
		args.put("clienteList", clienteList);

		openWindow("/gui/senda_amigo/editarCliente.zul",
				Labels.getLabel("editarClienteController.window.title"), args, MODAL);
	}

	private void refreshLista() {
		HibernateSearchObject<Cliente> sistemaBusqueda =
				new HibernateSearchObject<Cliente>(Cliente.class, pagingCliente.getPageSize());

		String numFidelidade = txtNumFidelidade.getValue();
		if (StringUtils.isNotBlank(numFidelidade)) {
			sistemaBusqueda.addFilterSome("listClienteFidelidad",
					Filter.and(Filter.equal("tarjetaFidelidad.numTarjeta", numFidelidade),
							Filter.equal("tarjetaFidelidad.activo", true)));
		} else {
			String nombcliente = txtNombCliente.getText();
			if (nombcliente != null  && !nombcliente.isEmpty()) {
				sistemaBusqueda.addFilterOr(Filter.ilike("nombcliente", "%" + nombcliente.trim().concat("%")), Filter.isNull("nombcliente"));
			}

			String apellidopaterno = txtApelidoPaterno.getText();
			if (apellidopaterno != null  && !apellidopaterno.isEmpty()) {
				sistemaBusqueda.addFilterOr(Filter.ilike("apellidopaterno", "%" + apellidopaterno.trim().concat("%")), Filter.isNull("apellidopaterno"));
			}

			String apellidomaterno = txtApelidoMaterno.getText();
			if (apellidomaterno != null && !apellidomaterno.isEmpty()) {
				sistemaBusqueda.addFilterOr(Filter.ilike("apellidomaterno", "%" + apellidomaterno.trim().concat("%")), Filter.isNull("apellidomaterno"));
			}
			
			String doc = txtNumRfc.getText();
			if (doc != null && !doc.isEmpty()) {
				sistemaBusqueda.addFilterOr(Filter.ilike("numIdentificaUno", "%" + doc.trim().concat("%")),Filter.ilike("numIdentificaDos", "%" + doc.trim().concat("%")));
			}
			
			if(cmbTipoIdentificacion.getSelectedItem() != null) {
				TipoIdentificacion tipoIdentificacion = (TipoIdentificacion) cmbTipoIdentificacion.getSelectedItem().getValue();
				sistemaBusqueda.addFilterOr(Filter.equal("tipoIdentificacionUno", tipoIdentificacion),Filter.equal("tipoIdentificacionDos", tipoIdentificacion));
			}
			
			String desccorreo = txtDesccorreo.getText();
			if (StringUtils.isNotBlank(desccorreo)) {
				sistemaBusqueda.addFilterOr(Filter.ilike("desccorreo", "%" + desccorreo.trim().concat("%")),Filter.ilike("desccorreo", "%" + desccorreo.trim().concat("%")));
			}
		}

		sistemaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);
		sistemaBusqueda.addSortAsc("nombcliente");
		sistemaBusqueda.addFilterNotEqual("clienteId", -1);

		plwCliente.init(sistemaBusqueda, clienteList, pagingCliente);

		if (clienteList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaClienteController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	public void onClick$btnPesquisa(Event ev) {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) {
		verCliente(new Cliente());
	}

	public void onChanging$txtNumFidelidade(Event ev) {
		txtNombCliente.setValue("");
		txtApelidoPaterno.setValue("");
		txtApelidoMaterno.setValue("");
	}

	public List<TipoIdentificacion> getLsTipoIdentificacion() {
		return lsTipoIdentificacion;
	}

	public void setLsTipoIdentificacion(List<TipoIdentificacion> lsTipoIdentificacion) {
		this.lsTipoIdentificacion = lsTipoIdentificacion;
	}
}
